@@ -77,6 +77,7 @@ int
 _gdbm_snapshot (GDBM_FILE dbf)
 {
   int s;	     /* snapshot file descriptor */
+  int oldsnap;       /* previous snapshot file descriptor */
   
   if (dbf->snapfd[0] < 0)
     /* crash consistency hasn't been requested on this database */
@@ -93,7 +94,8 @@ _gdbm_snapshot (GDBM_FILE dbf)
   
   s = dbf->snapfd[dbf->eo];
   dbf->eo = !dbf->eo;
-
+  oldsnap = dbf->snapfd[dbf->eo];
+  
   /* says "DON'T recover from this snapshot, writing in progress " */
   if (fchmod (s, S_IWUSR)) 
     {
@@ -141,6 +143,26 @@ _gdbm_snapshot (GDBM_FILE dbf)
       GDBM_SET_ERRNO (dbf, GDBM_FILE_SYNC_ERROR, FALSE);
       return -1;
     }
+
+  /*
+   * Mark the previous snapshot file write-only, indicating thereby
+   * that it contains obsolete data.  The point of this additional
+   * operation is to reduce the time window during which a crash would
+   * leave two readable snapshot files.  
+   */
+  if (fchmod (oldsnap, S_IWUSR))
+    {
+      GDBM_SET_ERRNO (dbf, GDBM_ERR_FILE_MODE, FALSE);
+      return -1;
+    }
+
+  /* commit permission bits */
+  if (fsync (oldsnap))
+    {
+      GDBM_SET_ERRNO (dbf, GDBM_FILE_SYNC_ERROR, FALSE);
+      return -1;
+    }
+  
   return 0;
 }
 
@@ -211,36 +233,97 @@ timespec_cmp (struct timespec const *a, struct timespec const *b)
   return 0;
 }
 
+static int
+check_snapshot_mode (int mode)
+{
+  if (!S_ISREG (mode))	/* file is not a regular file */
+    return -1;
+  if (S_IXUSR & mode)	/* file is executable */
+    return -1;
+  if (S_IRUSR & mode)
+    {
+      if (S_IWUSR & mode)
+	return -1;	/* file is both readable and writable */
+    }
+  else if (!(S_IWUSR & mode))
+    return -1;		/* file is neither readable nor writable */
+  /* All OK */
+  return 0;
+}
+
 static int
 stat_snapshot (const char *f, struct stat *st)
 {
   if (stat (f, st))
     return -1;
-  if (!S_ISREG (st->st_mode))	/* f is not a regular file */
-    return -1;
-  if (S_IXUSR & st->st_mode)	/* f is executable */
-    return -1;
-  if (S_IRUSR & st->st_mode)
+  if (check_snapshot_mode (st->st_mode))
     {
-      if (S_IWUSR & st->st_mode)
-	return -1;	/* f is both readable and writable */
+      errno = EACCES;
+      return -1;
+    }
+  return 0;
+}
+
+static int
+gdbm_numsync (const char *dbname, unsigned *numsync)
+{
+  GDBM_FILE dbf;
+  int rc = -1;
+  
+  dbf = gdbm_open (dbname, 0, GDBM_READER, S_IRUSR, NULL);
+  if (dbf)
+    {
+      if (dbf->xheader)
+	{
+	  *numsync = dbf->xheader->numsync;
+	  rc = 0;
+	}
+      gdbm_close (dbf);
+    }
+  return rc;
+}
+
+/*
+ * Return:
+ *   0  both numsyncs equal or result undefined
+ *  -1  a's numsync is one less than b's
+ *  -2  a's numsync is less than b's
+ *  +1  a's numsync is one greater than b's
+ *  +2  a's numsync is greater than b's
+ *
+ * Takes into account integer overflow. 
+ */
+ 
+static int
+gdbm_numsync_cmp (const char *a, const char *b)
+{
+  unsigned na, nb;
+
+  if (gdbm_numsync (a, &na) == 0 &&
+      gdbm_numsync (b, &nb) == 0)
+    {
+      if (na == UINT_MAX && nb == 0)
+	return -1;
+      else if (na == 0 && nb == UINT_MAX)
+	return 1;
+      else if (na < nb)
+	return na + 1 == nb ? -1 : -2;
+      else if (na > nb)
+	return na == nb + 1 ? 1 : 2;
     }
-  else if (!(S_IWUSR & st->st_mode))
-    return -1;		/* f is neither readable nor writable */
   return 0;
 }
 
 /*
  * Selects among the two given snapshot files the one to be used for
- * post-crash recovery and stores its value in *RET.
- * Returns 0 (GDBM_SNAPSHOT_OK) on success, GDBM_SNAPSHOT_ERR on error (examine
- * errno) and GDBM_SNAPSHOT_SAME in the unlikely case when two snapshots have
- * the same modification time.
+ * post-crash recovery.
+ * Returns one of the GDBM_SNAPSHOT_* constants (see gdbm.h).
+ * If GDBM_SNAPSHOT_OK is returned a pointer to the most recent snapshot
+ * name is stored in *ret.  Otherwise, *ret is untouched.
  */
 int
 gdbm_latest_snapshot (const char *even, const char *odd, const char **ret)
 {
-  int r, sum;
   struct stat st_even, st_odd;
   
   if (!ret || !even || !odd || strcmp (even, odd) == 0)
@@ -256,31 +339,57 @@ gdbm_latest_snapshot (const char *even, const char *odd, const char **ret)
 
   if (st_even.st_mode & S_IRUSR)
     {
+      int rc = GDBM_SNAPSHOT_OK;
+      
       if (!(st_odd.st_mode & S_IRUSR))
 	{
 	  *ret = even;
 	  return GDBM_SNAPSHOT_OK;
 	}
 
-      /*
-       * Both readable: check mtime.
-       * Select the newer snapshot, i.e. the one whose mtime
-       * is greater than the other's
+      /* Both readable: compare numsync value in the extended header.
+       * Select the snapshot with greater numsync value.
        */
-      switch (timespec_cmp (&st_even.st_mtim, &st_odd.st_mtim))
+      switch (gdbm_numsync_cmp (even, odd))
 	{
 	case -1:
 	  *ret = odd;
 	  break;
+
+	case -2:
+	  rc = GDBM_SNAPSHOT_SUSPICIOUS;
+	  break;
 	  
 	case 1:
 	  *ret = even;
 	  break;
+
+	case 2:
+	  rc = GDBM_SNAPSHOT_SUSPICIOUS;
+	  break;
 	  
-	case 0:
-	  /* Shouldn't happen */
-	  return GDBM_SNAPSHOT_SAME;
+	default:
+	  /*
+	   * Both readable: check mtime.
+	   * Select the newer snapshot, i.e. the one whose mtime
+	   * is greater than the other's
+	   */
+	  switch (timespec_cmp (&st_even.st_mtim, &st_odd.st_mtim))
+	    {
+	    case -1:
+	      *ret = odd;
+	      break;
+	      
+	    case 1:
+	      *ret = even;
+	      break;
+	      
+	    case 0:
+	      /* Shouldn't happen */
+	      rc = GDBM_SNAPSHOT_SAME;
+	    }
 	}
+      return rc;
     }
   else if (st_odd.st_mode & S_IRUSR)
     {
@@ -291,10 +400,9 @@ gdbm_latest_snapshot (const char *even, const char *odd, const char **ret)
     {      
       /* neither readable: this means the crash occurred during
 	 gdbm_failure_atomic() */
-      return GDBM_SNAPSHOT_BAD;
     }
 
-  return GDBM_SNAPSHOT_OK;
+  return GDBM_SNAPSHOT_BAD;
 }
 #else
 int
@@ -313,6 +421,31 @@ gdbm_latest_snapshot (const char *even, const char *odd, const char **ret)
 }
 #endif /* GDBM_FAILURE_ATOMIC */
 
+int
+gdbm_file_sync (GDBM_FILE dbf)
+{
+  int r = 0;  /* return value */
+#if HAVE_MMAP
+  r = _gdbm_mapped_sync (dbf);
+#elif HAVE_FSYNC
+  if (fsync (dbf->desc))
+    {
+      GDBM_SET_ERRNO (dbf, GDBM_FILE_SYNC_ERROR, TRUE);
+      r = 1;
+    }
+#else
+  sync ();
+  sync ();
+#endif
+#ifdef GDBM_FAILURE_ATOMIC
+  /* If and only if the conventional fsync/msync/sync succeeds,
+     attempt to clone the data file. */
+  if (r == 0)
+    r = _gdbm_snapshot (dbf);
+#endif /* GDBM_FAILURE_ATOMIC */
+  return r;
+}
+
 /* Make sure the database is all on disk. */
 
 int
@@ -324,6 +457,14 @@ gdbm_sync (GDBM_FILE dbf)
   /* Initialize the gdbm_errno variable. */
   gdbm_set_errno (dbf, GDBM_NO_ERROR, FALSE);
 
+  if (dbf->xheader)
+    {
+      dbf->xheader->numsync++;
+      dbf->header_changed = TRUE;
+    }
+  
+  _gdbm_end_update (dbf);
+  
   /* Do the sync on the file. */
   return gdbm_file_sync (dbf);
 }
