@@ -504,7 +504,7 @@ static const yytype_uint16 yyrline[] =
      127,   134,   138,   144,   150,   154,   169,   170,   177,   181,
      187,   191,   199,   200,   203,   203,   211,   215,   221,   222,
      225,   247,   251,   259,   263,   267,   272,   279,   283,   284,
-     287,   288,   291,   323,   351,   352,   355
+     287,   288,   291,   327,   355,   356,   359
 };
 #endif
 
@@ -1734,16 +1734,20 @@ yyreduce:
 		  lerror (&(yylsp[0]), _("%s is not a boolean variable"), varname);
 		  break;
 
+		case VAR_ERR_BADVALUE:
+		  lerror (&(yylsp[0]), _("%s: setting is not allowed"), (yyvsp[0].string));
+		  break;
+		  
 		default:
 		  lerror (&(yylsp[0]), _("unexpected error setting %s: %d"), (yyvsp[0].string), rc);
 		}
 	      free ((yyvsp[0].string));
 	    }
-#line 1743 "gram.c" /* yacc.c:1646  */
+#line 1747 "gram.c" /* yacc.c:1646  */
     break;
 
   case 53:
-#line 324 "gram.y" /* yacc.c:1646  */
+#line 328 "gram.y" /* yacc.c:1646  */
     {
 	      int rc = variable_set ((yyvsp[-2].string), VART_STRING, (yyvsp[0].string));
 	      switch (rc)
@@ -1769,11 +1773,11 @@ yyreduce:
 	      free ((yyvsp[-2].string));
 	      free ((yyvsp[0].string));
 	    }
-#line 1773 "gram.c" /* yacc.c:1646  */
+#line 1777 "gram.c" /* yacc.c:1646  */
     break;
 
   case 56:
-#line 356 "gram.y" /* yacc.c:1646  */
+#line 360 "gram.y" /* yacc.c:1646  */
     {
 	      int rc = variable_unset ((yyvsp[0].string));
 	      switch (rc)
@@ -1791,11 +1795,11 @@ yyreduce:
 		}
 	      free ((yyvsp[0].string));
 	    }
-#line 1795 "gram.c" /* yacc.c:1646  */
+#line 1799 "gram.c" /* yacc.c:1646  */
     break;
 
 
-#line 1799 "gram.c" /* yacc.c:1646  */
+#line 1803 "gram.c" /* yacc.c:1646  */
       default: break;
     }
   /* User semantic actions sometimes alter yychar, and that requires
@@ -2030,7 +2034,7 @@ yyreturn:
 #endif
   return yyresult;
 }
-#line 375 "gram.y" /* yacc.c:1906  */
+#line 379 "gram.y" /* yacc.c:1906  */
 
 
 void
@@ -2043,6 +2047,26 @@ terror (const char *fmt, ...)
   va_end (ap);
 }
 
+/* gdbmshell-specific version of gdbm_perror */
+void
+dberror (char const *fmt, ...)
+{
+  int ec = errno;
+  va_list ap;
+  
+  if (!interactive ())
+    fprintf (stderr, "%s: ", progname);
+  YY_LOCATION_PRINT (stderr, yylloc);
+  fprintf (stderr, ": ");
+  va_start (ap, fmt);
+  vfprintf (stderr, fmt, ap);
+  va_end (ap);
+  fprintf (stderr, ": %s", gdbm_strerror (gdbm_errno));
+  if (gdbm_check_syserr (gdbm_errno))
+    fprintf (stderr, ": %s", strerror (ec));
+  fputc ('\n', stderr);
+}
+
 int
 yyerror (char const *s)
 {
