@@ -34,7 +34,7 @@
 # define YY_YY_GRAM_H_INCLUDED
 /* Debug traces.  */
 #ifndef YYDEBUG
-# define YYDEBUG 0
+# define YYDEBUG 1
 #endif
 #if YYDEBUG
 extern int yydebug;
@@ -53,9 +53,10 @@ extern int yydebug;
     T_UNSET = 263,
     T_BOGUS = 264,
     T_CMD = 265,
-    T_NUM = 266,
-    T_IDENT = 267,
-    T_WORD = 268
+    T_SHELL = 266,
+    T_NUM = 267,
+    T_IDENT = 268,
+    T_WORD = 269
   };
 #endif
 /* Tokens.  */
@@ -67,16 +68,17 @@ extern int yydebug;
 #define T_UNSET 263
 #define T_BOGUS 264
 #define T_CMD 265
-#define T_NUM 266
-#define T_IDENT 267
-#define T_WORD 268
+#define T_SHELL 266
+#define T_NUM 267
+#define T_IDENT 268
+#define T_WORD 269
 
 /* Value type.  */
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
 
 union YYSTYPE
 {
-#line 49 "gram.y" /* yacc.c:1909  */
+#line 51 "gram.y" /* yacc.c:1909  */
 
   char *string;
   struct kvpair *kvpair;
@@ -90,7 +92,7 @@ union YYSTYPE
   struct { struct dsegm *head, *tail; } dsegmlist;
   struct command *cmd;
 
-#line 94 "gram.h" /* yacc.c:1909  */
+#line 96 "gram.h" /* yacc.c:1909  */
 };
 
 typedef union YYSTYPE YYSTYPE;
